#   This program demonstrates using an open office spreadsheet as a
#   repository for problems needing solution by GLPK

#  Nigel_Galloway@operamail.com
#  July 29th., 2008.
#

require 'rexml/parsers/pullparser'
require 'zip/zip'
require 'fusefs'

class KuKu_vfs
  def initialize(args)
    if (args.size != 2)
      puts "Usage: #{$0} <directory> <open office spreadsheet>"
      exit
    end
    @dirname = args.shift
    unless File.directory?(@dirname)
      puts "Usage: #{$0} <directory> <open office spreadsheet>"
      exit
    end
    @conts=[];
    @myF = args.shift;
    Zip::ZipFile.open(@myF).get_input_stream('content.xml'){|aFile|
      parser = REXML::Parsers::PullParser.new(aFile);
      while true do
        begin res = parser.pull; end until (res.start_element? and res[0] == 'table:table') or (res.end_element? and res[0] == 'office:body');
        if res.end_element? then break end;
        @conts += [res[1]['table:name']];
      end
    }
  end

  def mount
    FuseFS.set_root( self )
    # Mount under a directory given on the command line.
    FuseFS.mount_under @dirname
    FuseFS.run
  end

  def contents(path)
    @conts
  end

  def file?(path)
    @conts.each{|f|
      if path[1..-1] == f then return true end
    }
    return false
  end

  def parse_path(path)
    Zip::ZipFile.open(@myF).get_input_stream('content.xml'){|aFile|
      parser = REXML::Parsers::PullParser.new(aFile);
  		res = nil;
  		v   = nil;
	  	begin res = parser.pull; end until res.start_element? and res[0] == 'table:table' and res[1]['table:name'] == path[1..-1];
	  	rN = 0;
	  	begin
	  		res = parser.pull
	  		if res.start_element? and res[0] == 'table:table-row'
		  	  rN += 1;
	  			cN = 0;
	  			begin
		  			res = parser.pull
		  			if res.start_element? and res[0] == 'table:table-cell'
		  			  begin
			  				v = parser.pull
			  			end until (v.text?) or (v.end_element? and v[0] == 'table:table-cell')
			  			handleCell(rN,cN+=1,v);
			  			(1...res[1]['table:number-columns-repeated'].to_i).each{
			  				handleCell(rN,cN+=1,v);
			  			}
		  			end
		  		end until res.end_element? and res[0] == 'table:table-row';
	  		end
	  	end until res.end_element? and res[0] == 'table:table';
    }
  end

  def setFixed
    @cA = Array.new;
    @cC = Array.new;
    @cB = Array.new;
    @cE = Array.new;
    @cZ = Array.new;
  end

  def read_file(path)
    setFixed;
    parse_path(path);
    aString = sprintf("data;\nparam rows    := %d;\nparam cols    := %d;\nparam Zymbolz := %d;\nparam : Cblocks : Tstart   Tend :=\n",@rows,@cols,@Zymbolz);
    @cC.each{|r| aString << sprintf("%15d %8d %6d # %s\n",r.fetch('C'),r.fetch('start'),r.fetch('end'),r.fetch('comment'))};
    aString << ";\nparam :  Blocks : Cstart   Cend :=\n";
    @cB.each{|r| aString << sprintf("%15d %8d %6d # %s\n",r.fetch('C'),r.fetch('start'),r.fetch('end'),r.fetch('comment'))};
    aString << ";\nparam :   Cells :     X1     Y1      X2    Y2 :=\n";
    @cA.each{|r| aString << sprintf("%15d %8d %6d %7d %5d # %s\n",r.fetch('C'),r.fetch('X1'),r.fetch('Y1'),r.fetch('X2'),r.fetch('Y2'),r.fetch('comment'))};
    aString << ";\nparam :  Equalz : Qstart   Qend  Estart  Eend :=\n";
    @cE.each{|r| aString << sprintf("%15d %8d %6d %7d %5d # %s\n",r.fetch('C'),r.fetch('qstart'),r.fetch('qend'),r.fetch('estart'),r.fetch('eend'),r.fetch('comment'))};
    aString << ";\nparam :    Zumz :    Zum Zstart    Zend :=\n";
    @cZ.each{|r| aString << sprintf("%15d %8d %6d %7d # %s\n",r.fetch('C'),r.fetch('zum'),r.fetch('start'),r.fetch('end'),r.fetch('comment'))};
    aString << ";\nend;\n";
    return aString
  end

  def addCells(cNo,x1,y1,x2,y2,comment='')
    @cA<<{'C',cNo,'X1',x1,'Y1',y1,'X2',x2,'Y2',y2,'comment',comment};
  end

  def addCblock(cNo,tstart,tend,comment='')
    @cC<<{'C',cNo,'start',tstart,'end',tend,'comment',comment};
  end

  def addBlock(cNo,tstart,tend,comment='')
    @cB<<{'C',cNo,'start',tstart,'end',tend,'comment',comment};
  end

  def addZum(cNo,tstart,tend,zum,comment='')
    @cZ<<{'C',cNo,'zum',zum,'start',tstart,'end',tend,'comment',comment};
  end

  def addEqlz(cNo,qstart,qend,estart,eend,comment='')
    @cE<<{'C',cNo,'qstart',qstart,'qend',qend,'estart',estart,'eend',eend,'comment',comment};
  end

  def handleCell(rN,cN,v)
  end
end


